@@ -1,5 +1,6 @@
 import logging
 from six.moves import http_client
+from ..constants import DEFAULT_SWARM_ADDR_POOL, DEFAULT_SWARM_SUBNET_SIZE
 from .. import errors
 from .. import types
 from .. import utils
@@ -82,7 +83,9 @@ class SwarmApiMixin(object):
 
     @utils.minimum_version('1.24')
     def init_swarm(self, advertise_addr=None, listen_addr='0.0.0.0:2377',
-                   force_new_cluster=False, swarm_spec=None):
+                   force_new_cluster=False, swarm_spec=None,
+                   default_addr_pool=None, subnet_size=None,
+                   data_path_addr=None):
         """
         Initialize a new Swarm using the current connected engine as the first
         node.
@@ -107,9 +110,17 @@ class SwarmApiMixin(object):
             swarm_spec (dict): Configuration settings of the new Swarm. Use
                 ``APIClient.create_swarm_spec`` to generate a valid
                 configuration. Default: None
+            default_addr_pool (list of strings): Default Address Pool specifies
+                default subnet pools for global scope networks. Each pool
+                should be specified as a CIDR block, like '10.0.0.0/8'.
+                Default: None
+            subnet_size (int): SubnetSize specifies the subnet size of the
+                networks created from the default subnet pool. Default: None
+            data_path_addr (string): Address or interface to use for data path
+                traffic. For example, 192.168.1.1, or an interface, like eth0.
 
         Returns:
-            ``True`` if successful.
+            (str): The ID of the created node.
 
         Raises:
             :py:class:`docker.errors.APIError`
@@ -119,15 +130,44 @@ class SwarmApiMixin(object):
         url = self._url('/swarm/init')
         if swarm_spec is not None and not isinstance(swarm_spec, dict):
             raise TypeError('swarm_spec must be a dictionary')
+
+        if default_addr_pool is not None:
+            if utils.version_lt(self._version, '1.39'):
+                raise errors.InvalidVersion(
+                    'Address pool is only available for API version >= 1.39'
+                )
+            # subnet_size becomes 0 if not set with default_addr_pool
+            if subnet_size is None:
+                subnet_size = DEFAULT_SWARM_SUBNET_SIZE
+
+        if subnet_size is not None:
+            if utils.version_lt(self._version, '1.39'):
+                raise errors.InvalidVersion(
+                    'Subnet size is only available for API version >= 1.39'
+                )
+            # subnet_size is ignored if set without default_addr_pool
+            if default_addr_pool is None:
+                default_addr_pool = DEFAULT_SWARM_ADDR_POOL
+
         data = {
             'AdvertiseAddr': advertise_addr,
             'ListenAddr': listen_addr,
+            'DefaultAddrPool': default_addr_pool,
+            'SubnetSize': subnet_size,
             'ForceNewCluster': force_new_cluster,
             'Spec': swarm_spec,
         }
+
+        if data_path_addr is not None:
+            if utils.version_lt(self._version, '1.30'):
+                raise errors.InvalidVersion(
+                    'Data address path is only available for '
+                    'API version >= 1.30'
+                )
+            data['DataPathAddr'] = data_path_addr
+
         response = self._post_json(url, data=data)
-        self._raise_for_status(response)
-        return True
+        return self._result(response, json=True)
 
     @utils.minimum_version('1.24')
     def inspect_swarm(self):
@@ -165,7 +205,7 @@ class SwarmApiMixin(object):
 
     @utils.minimum_version('1.24')
     def join_swarm(self, remote_addrs, join_token, listen_addr='0.0.0.0:2377',
-                   advertise_addr=None):
+                   advertise_addr=None, data_path_addr=None):
         """
         Make this Engine join a swarm that has already been created.
 
@@ -176,7 +216,7 @@ class SwarmApiMixin(object):
             listen_addr (string): Listen address used for inter-manager
                 communication if the node gets promoted to manager, as well as
                 determining the networking interface used for the VXLAN Tunnel
-                Endpoint (VTEP). Default: ``None``
+                Endpoint (VTEP). Default: ``'0.0.0.0:2377``
             advertise_addr (string): Externally reachable address advertised
                 to other nodes. This can either be an address/port combination
                 in the form ``192.168.1.1:4567``, or an interface followed by a
@@ -184,6 +224,8 @@ class SwarmApiMixin(object):
                 the port number from the listen address is used. If
                 AdvertiseAddr is not specified, it will be automatically
                 detected when possible. Default: ``None``
+            data_path_addr (string): Address or interface to use for data path
+                traffic. For example, 192.168.1.1, or an interface, like eth0.
 
         Returns:
             ``True`` if the request went through.
@@ -193,11 +235,20 @@ class SwarmApiMixin(object):
                 If the server returns an error.
         """
         data = {
-            "RemoteAddrs": remote_addrs,
-            "ListenAddr": listen_addr,
-            "JoinToken": join_token,
-            "AdvertiseAddr": advertise_addr,
+            'RemoteAddrs': remote_addrs,
+            'ListenAddr': listen_addr,
+            'JoinToken': join_token,
+            'AdvertiseAddr': advertise_addr,
         }
+
+        if data_path_addr is not None:
+            if utils.version_lt(self._version, '1.30'):
+                raise errors.InvalidVersion(
+                    'Data address path is only available for '
+                    'API version >= 1.30'
+                )
+            data['DataPathAddr'] = data_path_addr
+
         url = self._url('/swarm/join')
         response = self._post_json(url, data=data)
         self._raise_for_status(response)
@@ -355,8 +406,10 @@ class SwarmApiMixin(object):
         return True
 
     @utils.minimum_version('1.24')
-    def update_swarm(self, version, swarm_spec=None, rotate_worker_token=False,
-                     rotate_manager_token=False):
+    def update_swarm(self, version, swarm_spec=None,
+                     rotate_worker_token=False,
+                     rotate_manager_token=False,
+                     rotate_manager_unlock_key=False):
         """
         Update the Swarm's configuration
 
@@ -370,6 +423,8 @@ class SwarmApiMixin(object):
                 ``False``.
             rotate_manager_token (bool): Rotate the manager join token.
                 Default: ``False``.
+            rotate_manager_unlock_key (bool): Rotate the manager unlock key.
+                Default: ``False``.
 
         Returns:
             ``True`` if the request went through.
@@ -378,12 +433,20 @@ class SwarmApiMixin(object):
             :py:class:`docker.errors.APIError`
                 If the server returns an error.
         """
-
         url = self._url('/swarm/update')
-        response = self._post_json(url, data=swarm_spec, params={
+        params = {
             'rotateWorkerToken': rotate_worker_token,
             'rotateManagerToken': rotate_manager_token,
             'version': version
-        })
+        }
+        if rotate_manager_unlock_key:
+            if utils.version_lt(self._version, '1.25'):
+                raise errors.InvalidVersion(
+                    'Rotate manager unlock key '
+                    'is only available for API version >= 1.25'
+                )
+            params['rotateManagerUnlockKey'] = rotate_manager_unlock_key
+
+        response = self._post_json(url, data=swarm_spec, params=params)
         self._raise_for_status(response)
         return True
