@@ -153,16 +153,20 @@ class ServiceCollection(Collection):
             image (str): The image name to use for the containers.
             command (list of str or str): Command to run.
             args (list of str): Arguments to the command.
-            constraints (list of str): Placement constraints.
-            preferences (list of str): Placement preferences.
-            platforms (list of tuple): A list of platforms constraints
-                expressed as ``(arch, os)`` tuples
+            constraints (list of str): :py:class:`~docker.types.Placement`
+                constraints.
+            preferences (list of tuple): :py:class:`~docker.types.Placement`
+                preferences.
+            platforms (list of tuple): A list of platform constraints
+                expressed as ``(arch, os)`` tuples.
             container_labels (dict): Labels to apply to the container.
             endpoint_spec (EndpointSpec): Properties that can be configured to
                 access and load balance a service. Default: ``None``.
             env (list of str): Environment variables, in the form
                 ``KEY=val``.
             hostname (string): Hostname to set on the container.
+            init (boolean): Run an init inside the container that forwards
+                signals and reaps processes
             isolation (string): Isolation technology used by the service's
                 containers. Only used for Windows containers.
             labels (dict): Labels to apply to the service.
@@ -278,6 +282,7 @@ CONTAINER_SPEC_KWARGS = [
     'hostname',
     'hosts',
     'image',
+    'init',
     'isolation',
     'labels',
     'mounts',
